﻿using Syncfusion.UI.Xaml.Grid;
using Syncfusion.UI.Xaml.Grid.Cells;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Data;

namespace EditingAndDataValidation
{
    public class ComboBoxColumnExt : GridComboBoxColumn
    {

        public ComboBoxColumnExt()
        {
            SetCellType("ComboBox");
        }

        protected override Freezable CreateInstanceCore()
        {
            return new ComboBoxColumnExt();
        }
    }

    public class ComboBoxRendererExt : GridVirtualizingCellRenderer<TextBlock, ComboBox>
    {
        public ComboBoxRendererExt()
        {

        }

        protected override TextBlock OnCreateDisplayUIElement()
        {
            return new TextBlock();
        }

        public override object GetControlValue()
        {
            if (!HasCurrentCellState)
                return null;

            return CurrentCellRendererElement.GetValue(IsInEditing ? Selector.SelectedValueProperty : TextBlock.TextProperty);

        }

        private void InitializeEditBinding(ComboBox uiElement, GridColumn column)
        {
            var comboBoxColumn = (GridComboBoxColumn)column;
            var source = comboBoxColumn.ValueBinding as Binding;
            var bind = new Binding
            {               
                    Mode = BindingMode.TwoWay,
                Path = source.Path,               
                StringFormat = source.StringFormat,
                TargetNullValue = source.TargetNullValue,
                UpdateSourceTrigger = UpdateSourceTrigger.PropertyChanged,
                ValidatesOnExceptions = source.ValidatesOnExceptions,
                AsyncState = source.AsyncState,
                BindingGroupName = source.BindingGroupName,
                IsAsync = source.IsAsync,
                NotifyOnSourceUpdated = source.NotifyOnSourceUpdated,
                NotifyOnTargetUpdated = source.NotifyOnTargetUpdated,
                UpdateSourceExceptionFilter = source.UpdateSourceExceptionFilter,
                XPath = source.XPath
            };            
            uiElement.SetBinding(ComboBox.SelectedValueProperty, bind);

            var itemsSourceBinding = new Binding { Path = new PropertyPath("ItemsSource"), Mode = BindingMode.TwoWay, Source = comboBoxColumn };
            uiElement.SetBinding(ComboBox.ItemsSourceProperty, itemsSourceBinding);
            var displayMemberBinding = new Binding { Path = new PropertyPath("DisplayMemberPath"), Mode = BindingMode.TwoWay, Source = comboBoxColumn };
            uiElement.SetBinding(ComboBox.DisplayMemberPathProperty, displayMemberBinding);
            var selectedValuePathBinding = new Binding { Path = new PropertyPath("SelectedValuePath"), Mode = BindingMode.TwoWay, Source = comboBoxColumn };
            uiElement.SetBinding(ComboBox.SelectedValuePathProperty, selectedValuePathBinding);


            var staysOpenOnEditBinding = new Binding { Path = new PropertyPath("StaysOpenOnEdit"), Mode = BindingMode.TwoWay, Source = comboBoxColumn };
            uiElement.SetBinding(ComboBox.StaysOpenOnEditProperty, staysOpenOnEditBinding);
            var isEditableBinding = new Binding { Path = new PropertyPath("IsEditable"), Mode = BindingMode.TwoWay, Source = comboBoxColumn };
            uiElement.SetBinding(ComboBox.IsEditableProperty, isEditableBinding);                       
            var itemTemplateBinding = new Binding { Path = new PropertyPath("ItemTemplate"), Mode = BindingMode.TwoWay, Source = comboBoxColumn };
            uiElement.SetBinding(ComboBox.ItemTemplateProperty, itemTemplateBinding); 
        }

        public override void OnInitializeEditElement(DataColumnBase dataColumn, ComboBox uiElement, object dataContext)
        {
            GridColumn column = dataColumn.GridColumn;
            InitializeEditBinding(uiElement, column);
            var textAlignment = new Binding { Path = new PropertyPath("TextAlignment"), Mode = BindingMode.OneWay, Source = column, Converter = new TextAlignmentToHorizontalAlignmentConverter() };
            uiElement.SetBinding(Control.HorizontalContentAlignmentProperty, textAlignment);
            var verticalAlignment = new Binding { Path = new PropertyPath("VerticalAlignment"), Mode = BindingMode.TwoWay, Source = column };
            uiElement.SetBinding(Control.VerticalContentAlignmentProperty, verticalAlignment);           
        }

        public override void OnInitializeDisplayElement(Syncfusion.UI.Xaml.Grid.DataColumnBase dataColumn, TextBlock uiElement, object dataContext)
        {
            var column = dataColumn.GridColumn;
            var gridcolumn = column as GridComboBoxColumn;

            uiElement.SetBinding(TextBlock.TextProperty, column.DisplayBinding);
            var textAlignment = new Binding { Path = new PropertyPath("TextAlignment"), Mode = BindingMode.OneWay, Source = column, Converter = new TextAlignmentToHorizontalAlignmentConverter() };
            uiElement.SetBinding(Control.HorizontalAlignmentProperty, textAlignment);
            var verticalAlignment = new Binding { Path = new PropertyPath("VerticalAlignment"), Mode = BindingMode.TwoWay, Source = column };
            uiElement.SetBinding(Control.VerticalAlignmentProperty, verticalAlignment);
            uiElement.Margin = new Thickness(3,0,1,0);
        }

        //public override void OnUpdateDisplayBinding(DataColumnBase dataColumn, TextBlock uiElement, object dataContext)
        //{
        //    SetDisplayBinding(uiElement, dataColumn.GridColumn, dataContext);
        //}     

        private static void SetDisplayBinding(TextBlock element, GridColumn column, object dataContext)
        {
            var customColumn = (ComboBoxColumnExt)column;
            var binding = new Binding
            {
                Path = new PropertyPath(customColumn.MappingName),
                Mode = BindingMode.TwoWay,
                UpdateSourceTrigger = UpdateSourceTrigger.PropertyChanged,                
            };

            element.SetBinding(TextBlock.TextProperty, binding);

        }
    }
}
