#region Copyright Syncfusion Inc. 2001 - 2016
// Copyright Syncfusion Inc. 2001 - 2016. All rights reserved.
// Use of this code is subject to the terms of our license.
// A copy of the current license can be obtained at any time by e-mailing
// licensing@syncfusion.com. Any infringement will be prosecuted under
// applicable laws. 
#endregion
using Syncfusion.UI.Xaml.Grid.Cells;
using Syncfusion.Windows.Shared;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Syncfusion.UI.Xaml.Grid;
using EditingAndDataValidation.DataModel;

namespace EditingAndDataValidation
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {        
        public MainWindow()

        {
            InitializeComponent();
            this.Grid.CellRenderers.Remove("MultiColumnDropDown");
            this.Grid.CellRenderers.Add("MultiColumnDropDown", new CustomMultiColumnDropDownRenderer());
        }
    }
    
    public class CustomMultiColumnDropDownRenderer: GridCellMultiColumnDropDownRenderer
    {
        protected override SfMultiColumnDropDownControl OnCreateEditUIElement()
        {
            return new CustomMultiColumnDropDownControl(this.DataGrid);
        }

        protected override bool ShouldGridTryToHandleKeyDown(KeyEventArgs e)
        {            
            if (e.Key == Key.Enter)
            {
                var multiDropDown = (this.CurrentCellRendererElement as CustomMultiColumnDropDownControl);
                multiDropDown.AddItem(multiDropDown);
            }
            return base.ShouldGridTryToHandleKeyDown(e);
        }
    }

    public class CustomMultiColumnDropDownControl : SfMultiColumnDropDownControl
    {
        SfDataGrid DataGrid;
        public CustomMultiColumnDropDownControl(SfDataGrid dataGrid) : base()
        {
            DataGrid = dataGrid;
        }
        public override void OnApplyTemplate()
        {
            Editor = GetTemplateChild("PART_TextBox") as TextBox;
            if (Editor != null)
                Editor.LostFocus += Editor_LostFocus;
            base.OnApplyTemplate();

        }
        private void Editor_LostFocus(object sender, RoutedEventArgs e)
        {
            if (!string.IsNullOrEmpty(SearchText))
            {
                AddItem(this);
            }
        }

        internal void AddItem(SfMultiColumnDropDownControl multiDropDown)
        {
            var textBox = multiDropDown.GetType().GetProperty("Editor", System.Reflection.BindingFlags.NonPublic | System.Reflection.BindingFlags.Instance).GetValue(multiDropDown);
            var newValue = textBox != null ? (textBox as TextBox).Text : null;
            var underlyingItems = ((multiDropDown as CustomMultiColumnDropDownControl).DataGrid.DataContext as ViewModel).ProductList;

            if (underlyingItems != null)
            {
                bool newRecord = underlyingItems.Any(x => x.Code == newValue.ToString());
                if (!newRecord)
                {
                    underlyingItems.Add(new ProductInfo() { Code = newValue.ToString(), ID = underlyingItems.Count + 1 });
                    multiDropDown.SelectedItem = underlyingItems.Last();
                }
            }
        }
    }
}
